"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const checked_exceptions_1 = require("checked-exceptions");
const dbl_linked_list_ds_1 = require("dbl-linked-list-ds");
const Bailout_1 = require("../internals/Bailout");
/**
 * Thrown when run() is called twice during the same loop.
 */
exports.ForbiddenNestedRun = checked_exceptions_1.check('ForbiddenNestedRun', () => 'calling scheduler.run() inside a scheduled job is forbidden');
const DEFAULT_SCHEDULER_OPTIONS = {
    bailout: 100
};
/**
 * 1. The scheduler runs based on "ticks"
 * 2. A special value called "nextTick" is maintained.
 * 3. nextTick is increased when everything for the current tick is completed.
 * 4. Asap uses nextTick to schedule work.
 */
class TestScheduler {
    constructor(options) {
        this.nextTick = 1;
        this.time = 0;
        this.Q = new Map();
        this.jobCount = 0;
        this.isRunning = false;
        this.options = Object.assign({}, DEFAULT_SCHEDULER_OPTIONS, options);
    }
    asap(job) {
        this.jobCount++;
        return this.insert(this.nextTick, job);
    }
    delay(job, duration) {
        this.jobCount++;
        return this.insert(this.now() + duration, job);
    }
    now() {
        return this.time;
    }
    run() {
        if (!this.isRunning) {
            this.isRunning = true;
            const checker = Bailout_1.Bailout(this.options.bailout);
            while (this.jobCount > 0 && checker()) {
                this.tick();
            }
            this.isRunning = false;
        }
        else {
            throw new exports.ForbiddenNestedRun();
        }
    }
    runTo(n) {
        const checker = Bailout_1.Bailout(this.options.bailout);
        while (this.now() < n && checker()) {
            this.tick();
        }
    }
    runBy(n) {
        this.runTo(this.now() + n);
    }
    tick() {
        this.time++;
        this.flush();
        this.nextTick = this.time + 1;
    }
    getList(seq) {
        const ll = this.Q.get(seq);
        if (ll) {
            return ll;
        }
        const linkedList = new dbl_linked_list_ds_1.LinkedList();
        this.Q.set(seq, linkedList);
        return linkedList;
    }
    insert(time, job) {
        const list = this.getList(time);
        const id = list.add(job);
        return {
            cancel: () => {
                this.jobCount--;
                list.remove(id);
            }
        };
    }
    flush() {
        const checker = Bailout_1.Bailout(this.options.bailout);
        const tick = this.time;
        const qElement = this.Q.get(tick);
        while (qElement && qElement.length > 0 && checker()) {
            const headElement = qElement.head();
            if (headElement) {
                headElement.value.execute();
                qElement.remove(headElement);
                this.jobCount--;
            }
        }
        this.Q.delete(tick);
    }
}
exports.TestScheduler = TestScheduler;
